import argparse
import logging
import pathlib
from datetime import datetime

from ptt_article_parser import Article as ArticleParser
from ptt_article_parser.dir import DIR
from ptt_article_parser.rename import format_filename

from .ptt_bot import PTTBot

__version__ = "0.1.1"

class DummyDir(DIR):
    def __init__(self, article):
        super().__init__()
        self.article = article
        
    def getAuthor(self, _file):
        return self.article.sender
        
    def getTitle(self, _file):
        return self.article.title
        
    def getTime(self, _file):
        date = datetime.strptime(self.article.date, "%m/%d")
        date = datetime.today().replace(month=date.month, day=date.day)
        if date > datetime.today():
            date = date.replace(year=date.year - 1)
        return date

def main():
    parser = argparse.ArgumentParser(description="Backup PTT mail.")
    parser.add_argument(
        "-u", "--user", help="username, otherwise prompt for the value."
    )
    parser.add_argument(
        "-p", "--pass", dest="password", help="password, otherwise prompt for the value."
    )
    parser.add_argument(
        "-d", "--dest", default=".", help="save to dest. Default: %(default)r"
    )
    parser.add_argument(
        "-v", "--verbose", action="store_true", help="print verbose message."
    )
    parser.add_argument(
        "-f", "--filename-format",
        default="{index}. [{board}] {title} [{author}] ({time:%Y%m%d%H%M%S}).ans",
        help="filename format. Default: %(default)r"
    )
    range_group = parser.add_mutually_exclusive_group(required=True)
    range_group.add_argument(
        "-r", "--range", nargs=2, type=int, metavar=("START", "END"),
        help="specify a range (inclusive). Negative values and zeros are "
             "allowed, they are treated as (last_index + value) i.e. --range 0 "
             "0 would download the last mail."
    )
    range_group.add_argument("--all", action="store_true", help="download all")
    args = parser.parse_args()
    
    if args.verbose:
        logging.basicConfig(level="INFO")
    
    with PTTBot(args.user, args.password) as bot:
        print("Login success, try entering your mail box")
        with bot.enter_mail():
            last_index = bot.get_last_index()
            if args.range:
                start, end = args.range
                if start <= 0:
                    start += last_index
                if end <= 0:
                    end += last_index
            elif args.all:
                start = 1
                end = last_index
            else:
                raise TypeError("invalid range")
                
            dest = pathlib.Path(args.dest)
            dest.mkdir(parents=True, exist_ok=True)
            for i in range(start, end + 1):
                print("Fetching mail: {}".format(i))
                article = bot.get_article(i)
                content = article.to_bytes()
                article_parser = ArticleParser(content)
                filename = format_filename(
                    article=article_parser,
                    format=args.filename_format,
                    dir=DummyDir(article),
                    extra={"index": i}
                )
                dest.joinpath(filename).write_bytes(content)



import pyte.graphics

fg2code = {name: str(key).encode("latin-1") for key, name in pyte.graphics.FG_ANSI.items()}
bg2code = {name: str(key).encode("latin-1") for key, name in pyte.graphics.BG_ANSI.items()}

fg2code.update({"default": b"37"})
bg2code.update({"default": b"40"})

def code_to_ansi(codes):
    return b"\x1b[" + b";".join(codes) + b"m"

class ColorState:
    RESET = b"\x1b[m"
    
    def __init__(self, init=None):
        self.bold = False
        self.fg = "default"
        self.bg = "default"
        
        if init:
            self.bold = init.bold
            self.fg = init.fg
            self.bg = init.bg
            
    def __eq__(self, other):
        return (
            self.bold == other.bold and
            self.fg == other.fg and
            self.bg == other.bg
        )
    
    def is_default(self):
        return not self.bold and self.fg == "default" and self.bg == "default"
        
    def diff(self, other):
        if self.is_default():
            return ColorState.RESET # reset
        codes = []
        if self.bold != other.bold:
            if not self.bold: # remove bold
                codes.append(b"0")
                codes.append(fg2code[self.fg])
                codes.append(bg2code[self.bg])
                return code_to_ansi(codes)
            codes.append(b"1")
        if self.fg != other.fg:
            codes.append(fg2code[self.fg])
        if self.bg != other.bg:
            codes.append(bg2code[self.bg])
        return code_to_ansi(codes)
            
    def gen(self, bg=None, fg=None, bold=None):
        if bg is None:
            bg = self.bg
        if fg is None:
            fg = self.fg
        if bold is None:
            bold = self.bold
        return self.bg

def colored_sequence(chars):
    color = ColorState()
    for c in chars:
        if c is None:
            yield b" "
            continue
        next_color = ColorState(c)
        if color != next_color:
            yield next_color.diff(color)
            color = next_color
        yield c.data.encode("latin-1")
    if not color.is_default():
        yield ColorState.RESET
        
def chars_to_bytes(chars):
    return b"".join(colored_sequence(chars))



"""
A pyte stream that would work with bytes.
"""
import re
import pyte

def no_c1_pattern(rx):
    # pyte uses a single backslash to escape every characters
    pattern = re.sub(r"\\[\x9b\x9d]", "", rx.pattern)
    return re.compile(pattern)
        
class ByteStream(pyte.ByteStream):
    # pylint: disable=protected-access
    # https://github.com/selectel/pyte/issues/118
    _text_pattern = no_c1_pattern(pyte.ByteStream._text_pattern)
    
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.select_other_charset("@")
